/**
 * Programa   : Ferramenta AutoSSH para servir de apoio a Ferramenta SVarLog do SAESD, para login automtico
 * em servidores linux por meio do uso do protocolo Secure Shell, atravs do gerenciamento de chaves assimtricas
 * Objetivo: Logar automaticamente em servidores Linux pelo protocolo SSH, realizar o processo de automao nas diferentes verses do SSH.
 * Programador: Alessandro Dutra de Andrade - RGM: 022693
 * Data: 20/09/2016
 */

/*
*   Parametros de entrada:
*   Param1: Servidor
*   Param2: Usuario
*   Param3: Comando
*   Param4: Versao Protocolo SSH cliente
*   Param5: Versao Protocolo SSH Servidor
*
*   Return : arquivo de log contendo o resultado do comando acima informado (no formato Log_Data_Hora.txt).
*/
#include "AutoSSH.h"

void Inicializacao(int argc, char **argv)
{
    /*declarando variaveis*/
    AUTO_SSH ParametrosSSH, RegistroSSH;
    char Id_Conexao[3], Servidor[TAMANHO], AuxServidor[TAMANHO];
    char *Registro = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run\\AutoSSH";
    int Retorno;
	/*compara se os parametros esto corretos*/
	if (argc == 6)
    {
        /*recepo dos parametros e preenchimento da estrutura SSH*/
        strcpy(ParametrosSSH.Servidor, argv[1]);
        strcpy(ParametrosSSH.Usuario, argv[2]);
        strcpy(ParametrosSSH.Comando, argv[3]);
        strcpy(ParametrosSSH.VerCliente, argv[4]);
        strcpy(ParametrosSSH.VerServidor, argv[5]);
        /*salvo em variaveis auxiliares*/
        sprintf(AuxServidor, ParametrosSSH.Servidor, "%s");
        /*para identificar o cliente*/
        strcpy(ParametrosSSH.LigacaoSimbolica, "");
        strcpy(Id_Conexao, "");
        if ((strcmp(ToUpper(ParametrosSSH.VerCliente), "SSH1") == 0) || (strcmp(ToUpper(ParametrosSSH.VerCliente), "SSH-1") == 0))
        {
            strcat(Id_Conexao, "1");
            /*atribuo um comando de ligao simbolica*/
            strcat(ParametrosSSH.LigacaoSimbolica, "ssh");
        }
        else if ((strcmp(ToUpper(ParametrosSSH.VerCliente), "SSH2") == 0) || (strcmp(ToUpper(ParametrosSSH.VerCliente), "SSH-2") == 0))
        {
            strcat(Id_Conexao, "2");
            /*atribuo um comando de ligao simbolica*/
            strcat(ParametrosSSH.LigacaoSimbolica, "ssh2");
        }
        else if (strcmp(ToUpper(ParametrosSSH.VerCliente), "OPENSSH") == 0)
        {
            strcat(Id_Conexao, "3");
            /*atribuo um comando de ligao simbolica*/
            strcat(ParametrosSSH.LigacaoSimbolica, "ssh");
        }
        /*para identificar o servidor*/
        if ((strcmp(ToUpper(ParametrosSSH.VerServidor), "SSH1") == 0) || (strcmp(ToUpper(ParametrosSSH.VerServidor), "SSH-1") == 0))
            strcat(Id_Conexao, "1");
        else if ((strcmp(ToUpper(ParametrosSSH.VerServidor), "SSH2") == 0) || (strcmp(ToUpper(ParametrosSSH.VerServidor), "SSH-2") == 0))
            strcat(Id_Conexao, "2");
        else if (strcmp(ToUpper(ParametrosSSH.VerServidor), "OPENSSH") == 0)
            strcat(Id_Conexao, "3");
        printf("\nIdentificador: %s.\n\n", Id_Conexao);
        /*leio do registro do windows e retorno o resultado para a variavel por referencia*/
        RegistroSSH = LerRegistroSSH(Registro);
        /*retiro o ponto caso haja para comparar o endereo do servidor*/
        sprintf(Servidor, RetiraCaracter(ParametrosSSH.Servidor, '.'), "%s");
        sprintf(ParametrosSSH.Servidor, AuxServidor, "%s");
        /*verifico se ja foi gerado processo de autenticao por chaves*/
        if ((strcmp(RegistroSSH.Usuario, ParametrosSSH.Usuario) == 0) && (strcmp(RegistroSSH.VerCliente, ParametrosSSH.VerCliente) == 0)
           && (strcmp(RegistroSSH.Servidor, Servidor) == 0) && (strcmp(RegistroSSH.VerServidor, ParametrosSSH.VerServidor) == 0))
        {
           /*procedimento que retorna a saida para o usuario ou seja o arquivo de log contendo o resultado do comando efetuado*/
           ProcessaSaida(ParametrosSSH);
        }else
        {
            /*funo principal do programa onde esta a rotina de autenticacao por chaves assimetricas*/
            Retorno = gera_autenticacao(ParametrosSSH, Id_Conexao);
            if (Retorno == 0)
            {
                /*cria registro para armazenar os dados de geracao da autenticacao*/
                CriaRegistro(Registro);
                //verifica se o registro existe
                if (BuscaRegistro(Registro) == 0)
                {
                    AdicionaValorChave(Registro, "Usuario", ParametrosSSH.Usuario);
                    AdicionaValorChave(Registro, "Servidor", Servidor);
                    AdicionaValorChave(Registro, "VersaoSSHCliente", ParametrosSSH.VerCliente);
                    AdicionaValorChave(Registro, "VersaoSSHServidor", ParametrosSSH.VerServidor);
                }
                /*procedimento que retorna a saida para o usuario ou seja o arquivo de log contendo o resultado do comando efetuado*/
                ProcessaSaida(ParametrosSSH);
            }
        }
    }
    else
    {
        printf("Erro!\nUso: AutoSSH.exe Servidor Usuario Comando \"Versao SSH cliente\" \"Versao SSH servidor\".\n");
    }
}

/*rotina para gerar arquivo de identificaco*/
void Cria_Arquivo()
{
		FILE *arq;
		arq = fopen("identification","w+");
		fputs("#IdKey id_dsa_1024_a",arq);
		fclose(arq);
}


/*rotina para gerar , enviar e converter chaves*/
int gera_autenticacao(AUTO_SSH _SSH, char *Id_TipoConexao)
{
    char Comando[TAMANHO], Diretorio[TAMANHO], USUARIO_PC[TAMANHO];
    //ApagaChaveAutenticacao();
    if (atoi(Id_TipoConexao) == SSH1_PARA_SSH1)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        /*Cria_Arquivo();*/
        printf("Convertendo a chave...\n");
        /*no  necessario converter a chave pois os formatos ssh1 para ssh1 sao compativeis*/
        printf("Enviando a chave para o diretorio de chaves autorizadas...\n");
        /*envia chave para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh/id_dsa_1024_a.pub>> ~/.ssh/authorized_keys\"");
        ExecutaComando(Comando);
    }
    else if (atoi(Id_TipoConexao) == SSH1_PARA_SSH2)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        /*Cria_Arquivo();*/
        printf("Convertendo a chave...\n");
        /*converte as chaves no formato windows para linux*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.VerServidor);
        strcat(Comando, " \"ssh-keygen -i -f id_dsa_1024_a.pub> ~/.ssh2/id_dsa_1024_b.pub\"");
        ExecutaComando(Comando);
        printf("Enviando a chave convertida para o diretorio de chaves autorizadas...\n");
        /*envia chave convertida para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh2/id_dsa_1024_b.pub>> ~/.ssh2/authorization\"");
        ExecutaComando(Comando);
    }
    else if (atoi(Id_TipoConexao) == SSH1_PARA_OPENSSH)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        /*Cria_Arquivo();*/
        printf("Convertendo a chave...\n");
        /*converte as chaves no formato windows para linux*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"ssh-keygen -i -f id_dsa_1024_a.pub> ~/.ssh/id_dsa_1024_b.pub\"");
        ExecutaComando(Comando);
        printf("Enviando a chave convertida para o diretorio de chaves autorizadas...\n");
        /*envia chave convertida para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh/id_dsa_1024_b.pub>> ~/.ssh/authorized_keys\"");
        ExecutaComando(Comando);
    }
    else if (atoi(Id_TipoConexao) == SSH2_PARA_SSH2)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen2 -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp2 ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        Cria_Arquivo();
        /*no  necessario converter a chave pois o formato ssh2  compativel com ssh2*/
        printf("Enviando a chave para o diretorio de chaves autorizadas...\n");
        /*envia chave para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh2/id_dsa_1024_a.pub>> ~/.ssh2/authorization\"");
        ExecutaComando(Comando);
    }
    else if (atoi(Id_TipoConexao) == SSH2_PARA_SSH1)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen2 -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp2 ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        /*Cria_Arquivo();*/
        printf("Convertendo a chave...\n");
        /*converte as chaves no formato windows para linux*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"ssh-keygen -i -f id_dsa_1024_a.pub> ~/.ssh/id_dsa_1024_b.pub\"");
        ExecutaComando(Comando);
        printf("Enviando a chave convertida para o diretorio de chaves autorizadas...\n");
        /*envia chave convertida para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh/id_dsa_1024_b.pub>> ~/.ssh/authorization\"");
        ExecutaComando(Comando);
    }
    else if (atoi(Id_TipoConexao) == SSH2_PARA_OPENSSH)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen2 -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp2 ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        /*Cria_Arquivo();*/
        printf("Convertendo a chave...\n");
        /*converte as chaves no formato windows para linux*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"ssh-keygen -i -f id_dsa_1024_a.pub> ~/.ssh/id_dsa_1024_b.pub\"");
        ExecutaComando(Comando);
        printf("Enviando a chave convertida para o diretorio de chaves autorizadas...\n");
        /*envia chave convertida para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh/id_dsa_1024_b.pub>> ~/.ssh/authorized_keys\"");
        ExecutaComando(Comando);
    }
    else if (atoi(Id_TipoConexao) == OPENSSH_PARA_OPENSSH)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        Cria_Arquivo();
        /*no  necessario converter a chave pois o formato openssh  compativel com openssh*/
        printf("Enviando a chave para o diretorio de chaves autorizadas...\n");
        /*envia chave convertida para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh/id_dsa_1024_a.pub>> ~/.ssh/authorized_keys\"");
        ExecutaComando(Comando);
    }
    else if (atoi(Id_TipoConexao) == OPENSSH_PARA_SSH1)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        /*Cria_Arquivo();*/
        printf("Convertendo a chave...\n");
        /*converte as chaves no formato windows para linux*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"ssh-keygen -i -f id_dsa_1024_a.pub> ~/.ssh/id_dsa_1024_b.pub\"");
        ExecutaComando(Comando);
        printf("Enviando a chave convertida para o diretorio de chaves autorizadas...\n");
        /*envia chave convertida para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh/id_dsa_1024_b.pub>> ~/.ssh/authorization\"");
        ExecutaComando(Comando);
    }
    else if (atoi(Id_TipoConexao) == OPENSSH_PARA_SSH2)
    {
        printf("Por favor digite sua senha de usuario linux:\n\n");
        /*caso nao exista pasta .ssh no servidor entao crio*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " mkdir .ssh");
        /*executar comamdo*/
        ExecutaComando(Comando);
        /* pula linha*/
        /*Apaga o conteudo caso exista diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "del /Q \"\\Users\\");
        /*procedimento que busca o nome do usuario do pc atravs da funo da API do windows GetEnvironmentVariable*/
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        /*cria o diretorio do ssh Userkeys*/
        strcpy(Diretorio, "");
        strcat(Diretorio, "mkdir \"\\Users\\");
        GetUsuario(USUARIO_PC);
        strcat(Diretorio, USUARIO_PC);
        strcat(Diretorio, "\\Application Data\\SSH\\UserKeys\"");
        ExecutaComando(Diretorio);
        printf("\nGerando chaves.....\n");
        printf("Atencao, nao digite nada logo apos passphrase e em again. So pressione enter.\n");
        printf("Se quizer voce pode criar uma senha, digite-a apos passphrase,repita em again.\n");
        printf("OBS: Se criar uma senha tera que digita-la toda vez que acessar o linux.\n");
        /* gera as chaves publicas e privadas no formato dsa e d 1024 bits*/
        ExecutaComando("ssh-keygen -t dsa -b 1024");
        /* limpa tela*/
        /*ExecutaComando("cls");*/
        printf("\nPreparando para enviar chave para servidor...\n");
        printf("Digite sua senha de usuario linux:\n");
        /* envia a chave publica para o servidor  solicitando a senha de usuario linux*/
        /*monto a string do diretorio que esta as chaves de autenticao do SSH*/
        strcpy(Comando, "");
        strcat(Comando, "scp ");
        strcat(Comando, "\"/users/");
        strcat(Comando, USUARIO_PC);
        strcat(Comando, "/application data/ssh/userkeys/id_dsa_1024_a.pub\" -o\" allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, "\":/home/");
        strcat(Comando, _SSH.Usuario);
        ExecutaComando(Comando);
        /*chama a rotina que cria o arquivo identification para autenticao de chaves*/
        /*Cria_Arquivo();*/
        printf("Convertendo a chave...\n");
        /*converte as chaves no formato windows para linux*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"ssh-keygen -e -f ~/.ssh2/id_dsa.pub > ~/.ssh2/id_dsa_ssh2.pub\"");
        ExecutaComando(Comando);
        printf("Enviando a chave convertida para o diretorio de chaves autorizadas...\n");
        /*envia chave convertida para a pasta authorized_keys*/
        strcpy(Comando, "");
        strcat(Comando, _SSH.LigacaoSimbolica);
        strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
        strcat(Comando, _SSH.Usuario);
        strcat(Comando, " ");
        strcat(Comando, _SSH.Servidor);
        strcat(Comando, " \"cat ~/.ssh2/id_dsa_ssh2.pub>> ~/.ssh2/authorization\"");
        ExecutaComando(Comando);
    }
    printf("Setando permissoes ao diretorio e a chave...\n");
    /*executa parametros de permisso para no haver erros na autenticao*/
    strcpy(Comando, "");
    strcat(Comando, _SSH.LigacaoSimbolica);
    strcat(Comando, " -o allowedauthentications=keyboard-interactive -l");
    strcat(Comando, _SSH.Usuario);
    strcat(Comando, " ");
    strcat(Comando, _SSH.Servidor);
    strcat(Comando, " chmod 755 ~ | chmod 755 ~/.ssh | chmod 644 ~/.ssh/authorized_keys");
    ExecutaComando(Comando);
    return (0);/*retorna '0' para identificar que a autenticao foi gerada com sucesso*/
}

/*rotina que usa funo da api do windows para substituir programa auxiliar do windows "ExecutaComando"*/
int ExecutaComando(char Comando[TAMANHO]){
    char AuxComando[TAMANHO];
    strcpy(AuxComando, "");
    strcat(AuxComando, "cmd /c ");
    strcat(AuxComando, Comando);
    if (CriaProcesso(AuxComando) == 0){
        return (0);
    }else{
      return (1);
    }
}
/*Procedimento para verificar se o usuario tem um conexo ativa*/
/*
BOOL InternetCheckConnection (
 _In_ LPCTSTR lpszUrl,
 _In_ DWORD dwFlags,
 _In_ DWORD dwReserved
);
*/
/*void TestaConexaoInternet(){
    if (!InternetCheckConnection("https://www.google.com.br/", 1, 0))
    {
      printf( "Voce esta sem conexao com a Internet!\nImpossivel continuar. Erro: (%lu).\n", GetLastError() );
      exit(0);
    }
}*/
/*executa um comando com elevacao de usario como administrador*/
/*HINSTANCE ShellExecute(
  _In_opt_  HWND hwnd,
  _In_opt_  LPCTSTR lpOperation,
  _In_      LPCTSTR lpFile,
  _In_opt_  LPCTSTR lpParameters,
  _In_opt_  LPCTSTR lpDirectory,
  _In_      INT nShowCmd);*/
/*int ExecutaComandoElevado(char Comando[TAMANHO]){
    char AuxComando[TAMANHO];
    HINSTANCE ValorRetorno;
    strcpy(AuxComando, "");
    strcat(AuxComando, "cmd /c ");
    strcat(AuxComando, Comando);
    ValorRetorno = ShellExecute(NULL, "runas", "cmd.exe", AuxComando, NULL, 0);
    if((int)ValorRetorno > 32)
    {
      return 1;
    }
    else{
        return 0;
    }
}*/

void ApagaChaveAutenticacao(){
    char Diretorio[TAMANHO], Usuario[TAMANHO];
    strcpy(Diretorio, "");
    strcat(Diretorio, "c:/Users/");
    GetUsuario(Usuario);
    strcat(Diretorio, Usuario);
    strcat(Diretorio, "/Application Data/SSH/UserKeys/id_dsa_1024_a.pub");
    /*apaga a pasta userkeys sem pedir confirmacao de modo silencioso, essa pasta contem  as chaves geradas anteriormente , que sera apagada e gerada outra no lugar*/
    DeletaArquivo(Diretorio);
    strcpy(Diretorio, "");
    strcat(Diretorio, "c:/Users/");
    GetUsuario(Usuario);
    strcat(Diretorio, Usuario);
    strcat(Diretorio, "/Application Data/SSH/UserKeys/id_dsa_1024_a");
    DeletaArquivo(Diretorio);
}
/*BOOL WINAPI CreateProcess(
  _In_opt_     LPCTSTR lpApplicationName,
  _Inout_opt_  LPTSTR lpCommandLine,
  _In_opt_     LPSECURITY_ATTRIBUTES lpProcessAttributes,
  _In_opt_     LPSECURITY_ATTRIBUTES lpThreadAttributes,
  _In_         BOOL bInheritHandles,
  _In_         DWORD dwCreationFlags,
  _In_opt_     LPVOID lpEnvironment,
  _In_opt_     LPCTSTR lpCurrentDirectory,
  _In_         LPSTARTUPINFO lpStartupInfo,
  _Out_        LPPROCESS_INFORMATION lpProcessInformation
);*/
int CriaProcesso(char Comando[TAMANHO])
{
    STARTUPINFO SUInfo;
    PROCESS_INFORMATION ProcInfo;
    ZeroMemory( &SUInfo, sizeof(SUInfo) );
    ZeroMemory( &ProcInfo, sizeof(ProcInfo) );
    SUInfo.cb          = sizeof(SUInfo);
    SUInfo.dwFlags     = STARTF_USESHOWWINDOW;
    SUInfo.wShowWindow = SW_NORMAL;
    if (!CreateProcess(NULL, Comando, NULL, NULL, 0, 0, NULL, NULL, &SUInfo, &ProcInfo)){
      printf( "Falha na execucao do comando p/ CMD: %s. Erro = (%lu).\n", Comando, GetLastError());
      return (0);
    }else{
      WaitForSingleObject(ProcInfo.hProcess, INFINITE);
      CloseHandle(ProcInfo.hProcess);
      CloseHandle(ProcInfo.hThread);
      return 1;
    }
}
/*
BOOL WINAPI DeleteFile(
  _In_  LPCTSTR lpFileName
);
*/
void DeletaArquivo(char Arquivo[TAMANHO]){
    if (!DeleteFile(Arquivo)){
      printf( "Falha ao tentar deletar o Arquivo: %s\n. Erro = (%lu).\n", Arquivo, GetLastError());
    }
}

void GetUsuario(char *USUARIO)
{
    char user[TAMANHO];
    DWORD ret = GetEnvironmentVariable("USERNAME", user, sizeof(user));
    if (!ret || ret == ERROR_ENVVAR_NOT_FOUND)
    {
      printf("Falha ao retornar nome do usuario!\n");
    }
    strcpy(USUARIO, user);
}

/*
https://msdn.microsoft.com/pt-br/library/windows/desktop/ms724844(v=vs.85).aspx
LONG WINAPI RegCreateKeyEx(
  _In_       HKEY                  hKey,
  _In_       LPCTSTR               lpSubKey,
  _Reserved_ DWORD                 Reserved,
  _In_opt_   LPTSTR                lpClass,
  _In_       DWORD                 dwOptions,
  _In_       REGSAM                samDesired,
  _In_opt_   LPSECURITY_ATTRIBUTES lpSecurityAttributes,
  _Out_      PHKEY                 phkResult,
  _Out_opt_  LPDWORD               lpdwDisposition
);

https://msdn.microsoft.com/pt-br/library/windows/desktop/ms724923(v=vs.85).aspx
LONG WINAPI RegSetValueEx(
  _In_             HKEY    hKey,
  _In_opt_         LPCTSTR lpValueName,
  _Reserved_       DWORD   Reserved,
  _In_             DWORD   dwType,
  _In_       const BYTE    *lpData,
  _In_             DWORD   cbData
);

https://msdn.microsoft.com/pt-br/library/windows/desktop/ms724895(v=vs.85).aspx
LONG WINAPI RegOpenKey(
  _In_     HKEY    hKey,
  _In_opt_ LPCTSTR lpSubKey,
  _Out_    PHKEY   phkResult
);

https://msdn.microsoft.com/pt-br/library/windows/desktop/ms724911(v=vs.85).aspx
LONG WINAPI RegQueryValueEx(
  _In_        HKEY    hKey,
  _In_opt_    LPCTSTR lpValueName,
  _Reserved_  LPDWORD lpReserved,
  _Out_opt_   LPDWORD lpType,
  _Out_opt_   LPBYTE  lpData,
  _Inout_opt_ LPDWORD lpcbData
);

https://msdn.microsoft.com/pt-br/library/windows/desktop/ms724837(v=vs.85).aspx
LONG WINAPI RegCloseKey(
  _In_ HKEY hKey
);
*/
int AdicionaValorChave(char *Registro, char *NomeValor, char *DadoValor)
{
    HKEY chave;
    if(RegOpenKey(HKEY_LOCAL_MACHINE, Registro, &chave) != 0) // Abre a chave "ParametrosSSH"
    {
        MessageBox(0,"Ocorreu um erro ao abrir a chave!","Erro",0x10);
        return 1;
    }
    if (RegSetValueEx(chave, NomeValor, 0, REG_SZ,(LPBYTE)DadoValor, strlen(DadoValor)) != 0)
        MessageBox(0,"Ocorreu um erro ao escrever o valor!","Erro",0x10);

    RegCloseKey(chave); // fecha a chave

    return 0;
}

int CriaRegistro(char *Registro)
{
    HKEY chave;

    if(RegCreateKey(HKEY_LOCAL_MACHINE, Registro, &chave) != 0)
    {
        MessageBox(0,"Ocorreu um erro ao criar a chave!","Erro",0x10);
        return 1;
    }

    RegCloseKey(chave);

    return 0;
}

int BuscaRegistro(char *Registro)
{
    HKEY chave;

    if(RegOpenKey(HKEY_LOCAL_MACHINE, Registro, &chave) != 0)
    {
        MessageBox(0,"Ocorreu um erro ao abrir a chave!","Erro",0x10);
        return 1;
    }

    RegCloseKey(chave);

    return 0;
}

AUTO_SSH LerRegistroSSH(char *Registro)
{
    HKEY chave;
    AUTO_SSH Reg_SSH;
    char Servidor[TAMANHO], Usuario[TAMANHO], VerCliente[TAMANHO], VerServidor[TAMANHO];
	DWORD Tipo, Tamanho_Buffer = TAMANHO;

    if(RegOpenKey(HKEY_LOCAL_MACHINE, Registro, &chave) != 0)
    {
        MessageBox(0,"Ocorreu um erro ao abrir a chave!","Erro",0x10);
        strcpy(Reg_SSH.Usuario, "");
        strcpy(Reg_SSH.Servidor, "");
        strcpy(Reg_SSH.VerCliente, "");
        strcpy(Reg_SSH.VerServidor, "");
        strcpy(Reg_SSH.LigacaoSimbolica, "");
        return (Reg_SSH);
    }else
    {
        if(RegQueryValueEx(chave,"Usuario",0,&Tipo,(LPBYTE)Usuario,&Tamanho_Buffer) == 0)
        {
             strcpy(Reg_SSH.Usuario, Usuario);
        }else
        {
             strcpy(Reg_SSH.Usuario, "");
        }
        if(RegQueryValueEx(chave,"Servidor",0,&Tipo,(LPBYTE)Servidor,&Tamanho_Buffer) == 0)
        {
             strcpy(Reg_SSH.Servidor, Servidor);
        }else
        {
             strcpy(Reg_SSH.Servidor, "");
        }
        if(RegQueryValueEx(chave,"VersaoSSHServidor",0,&Tipo,(LPBYTE)VerServidor,&Tamanho_Buffer) == 0)
        {
             strcpy(Reg_SSH.VerServidor, strTrim(VerServidor));
        }else
        {
             strcpy(Reg_SSH.VerServidor, "");
        }
        if(RegQueryValueEx(chave,"VersaoSSHCliente",0,&Tipo,(LPBYTE)VerCliente,&Tamanho_Buffer) == 0)
        {
             strcpy(Reg_SSH.VerCliente, strTrim(VerCliente));
        }else
        {
             strcpy(Reg_SSH.VerCliente, "");
        }
    }
    RegCloseKey(chave);

    return (Reg_SSH);
}
/*http://stackoverflow.com/questions/656542/trim-a-string-in-c*/
char *strTrim(char *s)
{
    int ix, jx;
    int len;
    char *buf;
    len = strlen(s);  /* possibly should use strnlen */
    buf = (char *) malloc(strlen(s)+1);

    for(ix=0, jx=0; ix < len; ix++)
    {
       if(!isspace(s[ix]))
          buf[jx++] = s[ix];
        buf[jx] = '\0';
        strncpy(s, buf, jx);  /* always looks as far as the null, but who cares? */
    }
    free(buf);            /* no good leak goes unpunished */
    return s;             /* modifies s in place *and* returns it for swank */
 }

char *RetiraCaracter(char *str, char caracter)
{
    int i, j;
    char *Retorno;
    Retorno = (char *) malloc(strlen(str)+1);
    for(i = 0, j = 0; i < strlen(str); i++)
    {
        if (str[i] != caracter)
        {
          Retorno[j++] = str[i];
        }
    }
    Retorno[j] = '\0';
    sprintf(str, Retorno, "%s");
    free(Retorno);
    return (str);
}

char *ToUpper(char *str)
{
    int i;
    char *Retorno;
    Retorno = (char *) malloc(strlen(str)+1);
    for(i = 0; i < strlen(str) ; i++)
    {
        Retorno[i] = toupper(str[i]);
    }
    Retorno[i] = '\0';
    sprintf(str, Retorno, "%s");
    free(Retorno);
    return (str);
}

void ProcessaSaida(AUTO_SSH ParamSSH)
{
    /*declarao de variaveis*/
    char StrData[9], StrHora[9], Traco[1] = "-", Comando[TAMANHO];
    int retorno;
    /*funes da api que retorna a data e hora do computador local*/
    _strdate(StrData);
    _strtime(StrHora);
    /*para no dar erro de criacao de arquivo entao substituo caracteres '/' e ':' por trao*/
    StrData[2] = Traco[0];
    StrData[5] = Traco[0];
    StrHora[2] = Traco[0];
    StrHora[5] = Traco[0];
    strcpy(Comando, "");
    strcat(Comando, ParamSSH.LigacaoSimbolica);
    strcat(Comando, " ");
    strcat(Comando, ParamSSH.Usuario);
    strcat(Comando, "@");
    strcat(Comando, ParamSSH.Servidor);
    strcat(Comando, " \"");
    strcat(Comando, ParamSSH.Comando);
    strcat(Comando, " \" > Arq_Log_");
    strcat(Comando, StrData);
    strcat(Comando, "_");
    strcat(Comando, StrHora);
    strcat(Comando, ".txt");
    retorno = ExecutaComando(Comando);
    if (retorno == 1)
        MessageBox(NULL, "Arquivo de Log gerado com sucesso!!!", "Atencao", MB_OK | MB_ICONEXCLAMATION);
    else
        MessageBox(NULL, "Erro ao gerar Arquivo de Log!!!", "Atencao", MB_OK | MB_ICONEXCLAMATION);
}
